//  Copyright 2006 Hewlett-Packard Development Company, L.P.
//
//  An example of using the DECwindows DECw$Term_Port call to
//  create a DECterm terminal on an OpenVMS system, and then
//  create a process and connect it to the DECterm.  (This
//  same basic sequence can be used to run an application
//  under the (character cell) debugger or to invoke an 
//  application that uses SMG or other such terminal displays
//  from within a detached process, for instance.)
//
//  Author: Stephen Hoffman, HP OpenVMS Engineering
//
//
//  To build this:
//
//    $!To compile, link and run CREATE_DECTERM (in SYS$LOGIN:)
//    $ cc create_decterm
//    $ link create_decterm, sys$input/opt
//    sys$share:decw$xlibshr/share
//    sys$library:decw$dwtlibshr/share
//    sys$share:vaxcrtl/share
//    sys$share:decw$terminalshr/share
//    $! @remote_decterm mynode
//
//
#pragma module  CREATE_DECTERM

#include <descrip.h>
#include <lib$routines.h>
#include <pqldef.h>
#include <prcdef.h>
#include <ssdef.h>
#include <starlet.h>
#include <stsdef.h>

// To build and run:
//    $ cc CREATE_DECTERM
//    $ link CREATE_DECTERM,sys$input/option
//    sys$share:DECW$TERMINALSHR.EXE/share
//    $ run CREATE_DECTERM

// This routine is not declared in a currently-available library
extern int DECw$Term_Port(void *,...);

main( void )
  {
  char *Display;
  int RetStat;
  int StsFlg;
  int DbgTermLen = 0;
#define DBGTERMBUFLEN 50
  char DbgTermBuf[DBGTERMBUFLEN];
  $DESCRIPTOR( Customization,
"DECW$TERMINAL.iconName:\tDebugging Session\n\
DECW$TERMINAL.title:\tDebugging Session" );
  $DESCRIPTOR( Command, "SYS$SYSDEVICE:[HOFFMAN]MAIN.EXE" );
  struct dsc$descriptor DbgTerm;

  DbgTerm.dsc$w_length  = DBGTERMBUFLEN;
  DbgTerm.dsc$b_dtype   = DSC$K_DTYPE_T;
  DbgTerm.dsc$b_class   = DSC$K_CLASS_S;
  DbgTerm.dsc$a_pointer = DbgTermBuf;

  // Request creation of a DECterm display
  RetStat = DECw$Term_Port(
	    0,		    // display (use default)
	    0,		    // setup file (use default)
	    &Customization, // customization
	    &DbgTerm,	    // resulting device name
	    &DbgTermLen,    // resulting device name length
	    0,		    // controller (use default)
	    0,		    // char buff (use default)
	    0 );            // char change buff (default)
  if ( !$VMS_STATUS_SUCCESS (RetStat ))
    lib$signal( RetStat );

  DbgTerm.dsc$w_length  = DbgTermLen;

  // Create the process as detached.
  StsFlg = PRC$M_DETACH;

  // Now create the process
  RetStat = sys$creprc( 
	    0,		    // PID
	    &Command,	    // Image to invoke
            &DbgTerm,	    // Input
            &DbgTerm,	    // Output
            0, 0, 0, 0, 0, 0, 0,
            StsFlg );	    // Process creation flags
  if ( !$VMS_STATUS_SUCCESS( RetStat ))
    lib$signal( RetStat );

  return SS$_NORMAL;
  }

