//
// COPYRIGHT (c) 1992, 1997, 1998, 1999 BY
// DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.
// ALL RIGHTS RESERVED.
//
// THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED
// ONLY  IN  ACCORDANCE  OF  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE
// INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER
// COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY
// OTHER PERSON.  NO TITLE TO AND  OWNERSHIP OF THE  SOFTWARE IS  HEREBY
// TRANSFERRED.
//
// THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE
// AND  SHOULD  NOT  BE  CONSTRUED  AS A COMMITMENT BY DIGITAL EQUIPMENT
// CORPORATION.
//
// DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS
// SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.
//
//
//++
//  Facility:
//
//	Examples
//
//  Version: V1.3
//
//  Abstract:
//
//      GBLSEC.C -- global section example code
//
//	Creates a backing storage file, and maps a global section
//	into process virtual address space (twice), and then shows
//	basic section operations.
//
//  Author:
//	Stephen Hoffman
//
//  Creation Date:  2-Jan-1990
//
//  Modification History:
//
//	Stephen Hoffman	    remove unused "stubs"
//	V1.1 27-Aug-1997
//	Stephen Hoffman	    added sys$updsecw calls.
//	V1.2 07-Apr-1998
//	Stephen Hoffman	    added $set_security, $dgblsc calls.
//	V1.3 17-Sep-1999
//
//  Build Instructions:
//
//      $ CC/DECC/PREFIX=ALL GBLSEC
//      $ LINK GBLSEC
//      $ RUN GBLSEC
//
//
//

#include <descrip.h>
#include <lib$routines.h>
#include <ossdef.h>
#include <psldef.h>
#include <rms.h>
#include <secdef.h>
#include <ssdef.h>
#include <starlet.h>
#include <stdio.h>
#include <string.h>
#include <stsdef.h>
#include <unistd.h>

#define P0SPACE ((void*)0x0200)
#define GBLSECMAX 10
#define MAXIL 10

struct SecStructDef
  {
  int GblSec[GBLSECMAX];
  };

ChurnData( char *Lbl, struct SecStructDef *Base, int WriteFlag )
    {
    int i;
    int RetStat;

    //  Now read or write some data in the specified window...

    printf( "Global section <%s> base is located at %p\n", Lbl, Base );
    if ( WriteFlag )
      {
      printf( "Writing data into the global section ...\n" );
      for ( i = 0; i < GBLSECMAX; i++)
	Base->GblSec[i] = i;
      }
    else
      {
      printf( "Reading data from the global section ...\n" );
      for ( i = 0; i < GBLSECMAX; i++)
         printf( "  %s->GblSec[%d] = %d\n", Lbl, i, Base->GblSec[i] );
      }

    return SS$_NORMAL;
    }

main()
    {
    int RetStat;
    $DESCRIPTOR( SecDsc, "FACNAM_GLOBAL_SECTION_NAME" );
    $DESCRIPTOR( ClsDsc, "SYSTEM_GLOBAL_SECTION" );
    int i;
    unsigned short int Iosb[4];
    void *InAdr1[2] = {P0SPACE,P0SPACE};
    void *RetAdr1[2] = {NULL,NULL};
    void *InAdr2[2] = {P0SPACE,P0SPACE};
    void *RetAdr2[2] = {NULL,NULL};
    unsigned long int OwnerUIC = geteuid();
    unsigned short int ProtMask = 0x0ff00;
    struct ItemList3
        {
        short int ItemLength;
        short int ItemCode;
        void *ItemBuffer;
        void *ItemRetLen;
        } SetSecIL[MAXIL];
    struct SecStructDef *Sec1, *Sec2;
    unsigned long int GblFlags;


    GblFlags = (SEC$M_EXPREG | SEC$M_GBL | SEC$M_SYSGBL | SEC$M_PAGFIL);

    //  Create, zero, and map the global section...
    //  Then overlay a structure (Sec1) onto the global section...

    RetStat = sys$crmpsc( InAdr1, RetAdr1, PSL$C_USER, 
	GblFlags | SEC$M_DZRO, &SecDsc, 0, 0, 0, 1, 0, 0, 0 );
    if (!$VMS_STATUS_SUCCESS( RetStat ))
	lib$signal( RetStat );
    Sec1 = RetAdr1[0];


    //	map the global section.  Again.
    //  Then overlay a structure (Sec2) onto the global section...

    RetStat = sys$crmpsc( InAdr2, RetAdr2, PSL$C_USER, 
	GblFlags, &SecDsc, 0, 0, 0, 1, 0, 0, 0 );
    if (!$VMS_STATUS_SUCCESS( RetStat ))
	lib$signal( RetStat );
    Sec2 = RetAdr2[0];


    //  Now write some data into one "window"...  And then read the
    //  data back in from the other "window".

    ChurnData( "Section1", Sec1, 1 );
    ChurnData( "Section1", Sec1, 0 );
    ChurnData( "Section2", Sec2, 0 );

    //  Now flush out the contents of the section to disk...  Twice...
    //  Once for each of the two address ranges that we currently have 
    //  mapped into the global section...

    RetStat = sys$updsecw( RetAdr1, NULL, PSL$C_USER, 0,
        0, Iosb, NULL, NULL );
    if (!$VMS_STATUS_SUCCESS( RetStat ))
	lib$signal( RetStat );
    if (!$VMS_STATUS_SUCCESS( Iosb[0] ))
	lib$signal( Iosb[0] );

    RetStat = sys$updsecw( RetAdr2, NULL, PSL$C_USER, 0,
        0, Iosb, NULL, NULL );
    if (!$VMS_STATUS_SUCCESS( RetStat ))
	lib$signal( RetStat );
    if (!$VMS_STATUS_SUCCESS( Iosb[0] ))
	lib$signal( Iosb[0] );


    //  Reset the protection and the ownership...
    //
    //  The sys$set_protection service works with pagefile-backed
    //  sections, but manipulation of file-backed sections is not 
    //  permitted.  (You must unmap everything, reset the security
    //  attributes on the backing file, then remap everything.)

    i = 0;
    SetSecIL[i].ItemLength     = sizeof( int );
    SetSecIL[i].ItemCode       = OSS$_OWNER;
    SetSecIL[i].ItemBuffer     = &OwnerUIC;
    SetSecIL[i++].ItemRetLen   = NULL;
    SetSecIL[i].ItemLength     = sizeof( short int );
    SetSecIL[i].ItemCode       = OSS$_PROTECTION;
    SetSecIL[i].ItemBuffer     = &ProtMask;
    SetSecIL[i++].ItemRetLen   = NULL;
    SetSecIL[i].ItemLength     = 0;
    SetSecIL[i].ItemCode       = 0;
    SetSecIL[i].ItemBuffer     = NULL;
    SetSecIL[i++].ItemRetLen   = NULL;
  
    RetStat = sys$set_security( &ClsDsc, &SecDsc, NULL, 
      OSS$M_RELCTX, SetSecIL, 0, 0 );
    if (!$VMS_STATUS_SUCCESS( RetStat ))
	lib$signal( RetStat );
      

    //  And redisplay the contents of the section, as was
    //  maintained by the backing storage file...

    ChurnData( "Section2", Sec2, 0 );

    // Now delete the section virtual address space ranges...

    RetStat = sys$deltva( RetAdr1, NULL, PSL$C_USER );
    if (!$VMS_STATUS_SUCCESS( RetStat ))
	lib$signal( RetStat );
    RetStat = sys$deltva( RetAdr2, NULL, PSL$C_USER );
    if (!$VMS_STATUS_SUCCESS( RetStat ))
	lib$signal( RetStat );


    //  delete the global section...  (If we need to...)

    if ( GblFlags & SEC$M_PERM )
      {
      RetStat = sys$dgblsc( 0, &SecDsc, 0 );
      if (!$VMS_STATUS_SUCCESS( RetStat ))
        lib$signal( RetStat );
      }

    return SS$_NORMAL;
    }

