Path:
pa.dec.com!decwrl!nntp1.crl.com!newshub.cts.com!alpha.sky.net!uunet!in5.uu.net!nn
ntprelay.mathworks.com!news.kjsl.com!mvb.saic.com!macro32
From: "Jess C. Goodman" <GOODMAN@nospam.com>
Newsgroups: vmsnet.internals
Subject: Re: Alpha environmental variables
Message-ID: <01IQZHNHKH0K99DP4X@accuwx.com>
Date: Tue, 09 Dec 1997 20:57:40 +0000 (GMT)
Organization: Macro32<==>Vmsnet.Internals Gateway
X-Gateway-Source-Info: Mailing List
Lines: 246

I wrote a quick C program that uses the undocumented $GETENV and $SETENV
system services.  Thanks to those of you who explained them.  The program
source is attached below.

When I RUN the program on my various Alpha V6.2 systems I get this (its
default action is to dump out all known environmental variables): 

$ RUN AXPENV
AUTO_ACTION = BOOT
BOOT_DEV = SCSI 0 2000 0 2 200 0 0
BOOTCMD_DEV = SCSI 0 2000 0 2 200 0 0
BOOTED_DEV = SCSI 0 2000 0 2 200 0 0
BOOT_FILE = 
BOOTED_FILE = 
BOOT_OSFLAGS = 0,0
BOOTED_OSFLAGS = 0,0
BOOT_RESET = ON
DUMP_DEV = 
ENABLE_AUDIT = ON
LICENSE = MU
CHAR_SET = 
LANGUAGE = 6
TTY_DEV = 0

You can also specify the name or one of the variables on a command line
to get just its value, although this is no more useful then F$GETENV().

But when I try to use it with a command line that calls $SETENV to change
an environmental variable's value I get:

$ MCR []AXPENV AUTO_ACTION RESTART
%SYSTEM-F-NOT_LOADED, system service or exec routine is not loaded

So what gives?  Is the $SETENV service not really there in V6.2, or
is there some undocumented way to load this undocumented service?
Does this work on a 7.x system??

I want to change AUTO_ACTION from BOOT to RESTART on all my Alphas
(since I discovered that with it set as BOOT you will not get a
crash dump written after some bugcheks, for example kernel stack
not valid).  I was hoping to not have to reboot them all to do this.

But why I really wanted to be able to display/change the AXP console
environemtal variables was for the non-standard I/O driver variables,
such as EWx0_MODE.  You can change the speed and duplex setting of
a DE500 ethernet controller using the LANCP utility, but it doesn't
stick; the next time the system reboots it will go back to its old
EWx0_MODE console setting!

We've been testing various 100mb/sec capable ethernet switches, and
since we can't reboot in the middle of the day (or risk crashing by
using the system halt button and hoping to CONTINUE in time) we often
have to leave a system in a state where it will not reboot by itself
if it crashes due to its current EWx0_MODE state not having been saved.

The $GETENV/$SETENV interface uses item codes that are defined in
STENVDEF.H only for 17 standard environmental variables (the last
two are not supported on my systems).  As I look more closely at the
AXP Architecture Reference Manual documentation of the GET_ENV/SET_ENV
console callback routines, it seems that they too will only work with
these standard variables - you pass them in an ID #, not a string.

So I guess I'm SOL.

-------------------------------------------------------------------------------
Jess C. Goodman  (GOODMAN@ACCUWX.CxM)
"Any technology sufficiently advanced is indistinguishable from magic."  Clarke
-------------------------------------------------------------------------------
--------------------------Attachment-----------------------------------
/*
*******************************************************************************
**
** AXPENV.C
**
** This program shows/sets the console environmental variables on an Alpha.
** These variables are normally only accessed from the system console using
** SHOW/SET at the >>> prompt.
**
** Uses the undocumented/unsupported system services SYS$GETENV/SYS$SETENV.
**
** Environment:
**    OpenVMS AXP  (required version ???)
**    CMKRNL privilege is required for setting an environmental variable
**
** Usage:
** $ AXPENV := $SYS$DISK:[]AXPENV
**
** AXPENV			!shows all known enviromental variables
** AXPENV name_or_number	!shows the given enviromental variable
** AXPENV name_or_number string !sets given enviromental variable to "string"
**
** If a number is used as input it can be decimal (default), octal (leading 0)
** or hexadecimal (leading 0x).
**
** Note you can also use the undocumented lexical function F$GETENV("name") to
** display a given environmental variable.
**
** The supported environmental variables are defined in the system include file
** STENVDEF.H.  Each environmental variable has a number which is the item code
** passed to $GETENV/$SETENV.   Symbolic names for the variables start with the
** prefix "STENVDEF$_K_".  These symbolic names, minus the prefix, are accepted
** by F$GETENV and this program.  However to do that we had to hard code all of
** the known symbolic names in a list below; this item_name list can be updated
** as need be along with MAX_ITEM_CODE.  Any item number can be used whether or
** not it's defined below or in STENVDEF.  If the item number is not valid this
** returns SS$_BADPARAM.
**
** V1.0 Dec 97 Jess C. Goodman	- First pass
*******************************************************************************
*/

/*  ANSI C header files */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/* VMS system header files */
#include <stenvdef.h>	/* defines item codes for environmental variables */
#include <starlet.h>
#include <ssdef.h>

void lib$signal();
void lib$stop();

/* We must make an assumption about the value of the last known item code. */
/* If more item codes are added to STENVDEF then update this appropiately. */
#define MAX_ITEM_CODE STENVDEF$_K_BTFLAGS

main( int argc, char *argv[])
{
  int status;
  int retlen;
  int item;
  int itemin;
  char buf[81];
  char unknown_item_name[] = "id# xxxxx";
  char *charptr;

  /* Define the special quadword itemlist structure and initialize it.  */
  /* Concatenate ITEMTYPE structure defined in stenvdef.h with endlist. */
  /* Note the itemtype structure defines its pointers as a quadword but */
  /* uses an int array to do so, so init. the high-order longword to 0. */
  struct itemlist_tag { struct itemtype it; int endlist;};
  struct itemlist_tag itemlist={0};      /* used for $GETENV */
  struct itemlist_tag set_itemlist={0};  /* used for $SETENV */

  /* Define character strings that name all the supported item codes.  We */
  /* use these strings in the display of our show function, and allow the */
  /* user to enter them to specify a particular variable for set or show. */
  /* We do NOT assume that all codes from 1 -> MAX_ITEM_CODE are defined. */
  /* If more items are defined in a newer STENVDEF then up MAX_ITEM_CODE. */
  char *item_names[MAX_ITEM_CODE+1] = {NULL};

  item_names[STENVDEF$_K_AUTO_ACTION] = "AUTO_ACTION";
  item_names[STENVDEF$_K_BOOT_DEV] = "BOOT_DEV",
  item_names[STENVDEF$_K_BOOTCMD_DEV] = "BOOTCMD_DEV";
  item_names[STENVDEF$_K_BOOTED_DEV] =  "BOOTED_DEV";
  item_names[STENVDEF$_K_BOOT_FILE] = "BOOT_FILE";
  item_names[STENVDEF$_K_BOOTED_FILE] = "BOOTED_FILE";
  item_names[STENVDEF$_K_BOOT_OSFLAGS] = "BOOT_OSFLAGS";
  item_names[STENVDEF$_K_BOOTED_OSFLAGS] = "BOOTED_OSFLAGS";
  item_names[STENVDEF$_K_BOOT_RESET] = "BOOT_RESET";
  item_names[STENVDEF$_K_DUMP_DEV] = "DUMP_DEV";
  item_names[STENVDEF$_K_ENABLE_AUDIT] = "ENABLE_AUDIT";
  item_names[STENVDEF$_K_LICENSE] = "LICENSE";
  item_names[STENVDEF$_K_CHAR_SET] = "CHAR_SET";
  item_names[STENVDEF$_K_LANGUAGE] = "LANGUAGE";
  item_names[STENVDEF$_K_TTY_DEV] = "TTY_DEV";
  item_names[STENVDEF$_K_SYSROOT] = "SYSROOT";
  item_names[STENVDEF$_K_BTFLAGS] = "BTFLAGS";
 
  /* Are there any command line arguments?  The first argument can be */
  /* a number or a string to indicate which environmental variable we */
  /* are interested in. */
  itemin = 0;
  if (argc > 1) {

    itemin = strtol( argv[1], &charptr, 0);  /* check if an integer string */
    if (charptr != argv[1]+strlen(argv[1])) {		      /* integer? */
      /* No, loop thru known item codes comparing their names to user input */
      for (charptr = argv[1]; *charptr != '\0'; charptr++) {
        *charptr = toupper( *charptr);		 	/* upcase string */
      }
      for (itemin=1; itemin<=MAX_ITEM_CODE; itemin++) {
	if (item_names[itemin] != NULL) {
          if (strcmp( item_names[itemin], argv[1]) == 0) break;
	}
      }
      if (itemin > MAX_ITEM_CODE) lib$signal( SS$_BADPARAM);
    }
  }

  if (itemin > 0) {		/* Valid user-specified item name/number ? */
    item = itemin;		/* Yes that's the only item we will handle */

    /* If second command line argument use it as a new value for the item. */
    if (argc > 2) {
      for (charptr = argv[2]; *charptr != '\0'; charptr++) {
        *charptr = toupper( *charptr);			 /* upcase string */
      }
      set_itemlist.it.stenvdef$l_item_code = itemin;
      set_itemlist.it.stenvdef$l_buf_len = strlen(argv[2]);
      set_itemlist.it.stenvdef$q_buf_addr[0] = (int) argv[2];
      set_itemlist.it.stenvdef$q_ret_addr[0] = (int) &retlen; /* required */

      status = sys$setenv( &set_itemlist);
      if (!(status&1)) lib$stop(status);
    } /* If successful verify new value by using $GETENV to display it.  */
  }
  else {
    item = 1;	/* No valid input; display all items codes from #1 on up */
  }

  /* Note we don't check above/below if user's input was > MAX_ITEM_CODE. */
  /* He/she may know of a valid item code that's not defined in STENVDEF. */

  /* Display the current (or new) value of the environmental variable(s). */
  itemlist.it.stenvdef$l_buf_len = sizeof(buf)-1;
  itemlist.it.stenvdef$q_buf_addr[0] = (int) &buf;
  itemlist.it.stenvdef$q_ret_addr[0] = (int) &retlen;

  do {	/* So we always execute loop once even if user_input>MAX_ITEM_CODE */
    itemlist.it.stenvdef$l_item_code = item;
    status = sys$getenv( &itemlist);
    if (status&1) {
      buf[retlen] = '\0';
      if (item<=MAX_ITEM_CODE ? item_names[item]!=NULL : FALSE) {
        charptr = item_names[item];
      }
      else {
        sprintf( unknown_item_name, "item #%d", item);
        charptr = unknown_item_name;
      }
      printf( "%s = %s\n", charptr, buf);
    }
    else {		/* Our $GETENV call was not successful */
      /* If item code undefined go on to next item unless user requested it. */
      if (status == SS$_BADPARAM  && itemin<=0) continue;
      lib$signal(status);
    }
    if (itemin > 0) break;	  /* single iteration if user specified item */

  }
  while (item++ < MAX_ITEM_CODE); /* looping until past last known item code */
}
