#module	GSD_FETCH "SRH X1.0-000"
#pragma builtins

/*
** COPYRIGHT (c) 1992 BY
** DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.
** ALL RIGHTS RESERVED.
**
** THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED
** ONLY  IN  ACCORDANCE  OF  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE
** INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER
** COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY
** OTHER PERSON.  NO TITLE TO AND  OWNERSHIP OF THE  SOFTWARE IS  HEREBY
** TRANSFERRED.
**
** THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE
** AND  SHOULD  NOT  BE  CONSTRUED  AS A COMMITMENT BY DIGITAL EQUIPMENT
** CORPORATION.
**
** DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS
** SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.
*/

/*
**++
**  Facility:
**
**	Examples
**
**  Version: V1.0
**
**  Abstract:
**
**	Example of directly working with the global section list
**
**  Author:
**	Steve XDELTA:: Hoffman
**
**  Creation Date:  26-Jul-1991
**
**  Modification History:
**--
*/
/*
**  GSD_FETCH.C
**
**  Example routine that fetches the name of the first global section
**  in the global section pending-delete list.
**
**  Requires CMKRNL privilege, and <obviously> has the potential
**  to crash your system.  (It hasn't crashed my workstation.  Yet.)
**
**  The DCL command procedure used to extract the GSD definitions from
**  the macro libraries, CVT_MAR-TO-H.COM, is posted in the notesfile
**  TURRIS::VAXC, note 3293.3.
**
**  Have fun, don't run this program on a critical system, and let me
**  know if you encounter any problems.  (Though I doubt you will see
**  any problems...)
**
**  No kernel-mode interlocking (IPL or spinlocks) is performed.  It
**  is thus possible to encounter a "corrupted" GSD pointer, due to
**  alterations in the GSD linked list.  Ka-Boom!
**
**	$! Build procedure
**	$
**	$ CC GSD_FETCH
**	$
**	$ LINK GSD_FETCH,SYS$SYSTEM:SYS.STB/SELECT,SYS$INPUT/OPT
**	SYS$SHARE:VAXCRTL/SHARE
**	$
**
**
**  Author: Steve XDELTA:: Hoffman 26-Jul-1991
*/
#pragma	    builtins

#include    <stdio.h>
#include    <ssdef.h>
#include    <starlet.h>
#include    <stsdef.h>
/*
/* SRH:<SCRATCH>$GSDDEF.H;
/* cvt_mar-to-h.com V1.0-000
/* VMS V5.4
/* 23-OCT-1990 12:56:03.04
*/
#define GSD$M_VALID (1)
#define GSD$M_LOCKED (2)
#define GSD$M_DELPEND (4)
#define GSD$M_INITFAIL (8)
#define GSD$M_DUPGSD (16)
#define GSD$K_LENGTH (39)
#define GSD$C_LENGTH (39)
#define GSD$K_EXTGSDLNG (53)
#define GSD$C_EXTGSDLNG (53)
#define GSD$S_GSDDEF (53)
#define GSD$L_GSDFL (0)
#define GSD$V_VALID (0)
#define GSD$V_LOCKED (1)
#define GSD$V_DELPEND (2)
#define GSD$V_INITFAIL (3)
#define GSD$V_DUPGSD (4)
#define GSD$L_GSDBL (4)
#define GSD$W_SIZE (8)
#define GSD$B_TYPE (10)
#define GSD$B_HASH (11)
#define GSD$L_PCBUIC (12)
#define GSD$W_PCBGRP (14)
#define GSD$L_FILUIC (16)
#define GSD$W_PROT (20)
#define GSD$W_GSTX (22)
#define GSD$L_IDENT (24)
#define GSD$L_ORB (28)
#define GSD$L_IPID (32)
#define GSD$W_FLAGS (36)
#define GSD$T_GSDNAM (38)
#define GSD$L_BASEPFN (40)
#define GSD$L_PAGES (44)
#define GSD$L_REFCNT (48)
#define GSD$T_PFNGSDNAM (52)
#define GSD$C_PFNBASMAX (4)
#define GSD$K_SHMGSDLNG (120)
#define GSD$C_SHMGSDLNG (120)
#define GSD$S_GSDDEF1 (124)
#define GSD$B_LOCK (84)
#define GSD$B_PROCCNT (85)
#define GSD$B_CREATPORT (86)
#define GSD$B_DELETPORT (87)
#define GSD$L_BASPFN1 (88)
#define GSD$L_BASCNT1 (92)
#define GSD$L_PTECNT1 (120)
/*
/* End SRH:<SCRATCH>$GSDDEF.H;
*/

globalref int *EXE$GL_GSDDELFL, *EXE$GL_GSDGRPFL, *EXE$GL_GSDMTX;
int gsd_getnam_k( char *, char * );

main()
    {
    char *gsd;
    char buffer[32+1];
    int arglist[3];
    unsigned long int retstat;
    unsigned long int ascic;

    /*
    **	Get the address of the first GSD.
    **
    **	(Right now, the GSD pointer is in user-read (the page protection
    **	is URKW or UREW) memory, so we can get at it.  The GSDs themselves,
    **	however, are not located in user-read memory.)
    */
    gsd = EXE$GL_GSDDELFL;

    /*
    **	Build the argument list...
    */
    arglist[0] = 2;
    arglist[1] = gsd;
    arglist[2] = buffer;

    retstat = sys$cmkrnl( gsd_getnam_k, arglist );
    if (!$VMS_STATUS_SUCCESS( retstat ))
	return retstat;

    /*
    **	The string returned in the buffer is in counted-ASCII (ASCIC)
    **	format, so we'll need to massage it a little before we print it.
    */
    ascic = buffer[0];
    buffer[(ascic + 1)] = 0;
    
    printf("%*s", ascic, (buffer + 1) );

    return SS$_NORMAL;
    }

/*
**  Kernel mode routine that snags a field from the specified GSD.
**  Can be modified to return the forward link, if chaining through
**  the GSDs becomes required.
*/
int 
gsd_getnam_k( char *gsd, char *buffer )
    {
    /*
    **	We're in kernel mode, so do no more than we have to, and do not
    **	call anything in any run-time libraries -- CRTL or otherwise.
    **
    **	...Here we use the MOVC5 built-in to avoid calling the CRTL.
    */
    if ( ((int) gsd ) >= 0 )
	return SS$_BADPARAM;

    _MOVC5( 32, (gsd + GSD$T_GSDNAM), 0, 32+1, buffer );

    return SS$_NORMAL;
    }
