#include <descrip.h>
#include <fibdef.h>
#include <iodef.h>
#include <lib$routines.h>
#include <ssdef.h>
#include <starlet.h>
#include <stdio.h>
#include <string.h>
#include <stsdef.h>

/*
//	Be careful when working with this program, as it locks the
//	target volume.  Do not allow the program to exit without
//	issuing the unlock request or without leaving yourself
//	another way to issue an unlock request, as it will leave
//	the volume in a writelocked state.
//
//	Production code should include <CTRL/Y> and exit handlers
//	that bring the volume into a known state, and unlock it.
//
//	Failure to exercise caution may leave the volume locked.
//	(And it will remain locked across reboots!)
//
//	Stephen Hoffman,
//	shoffman@star.enet.dec.com
//	Digital Equipment Corporation
*/
main( int argc, char **argv, char **envp )
    {
    $DESCRIPTOR( device, "TargetDevice:" );
    short int sIoChannel;
    short int qIosb[4];
    long int lStatus;
    struct fibdef1 sFib;
    struct dsc$descriptor_s dFib;

    /*
    //	Set up the fib descriptor...
    */
    dFib.dsc$w_length  = sizeof( struct fibdef1 );
    dFib.dsc$a_pointer = (char *) &sFib;
    dFib.dsc$b_class   = DSC$K_CLASS_S;
    dFib.dsc$b_dtype   = DSC$K_DTYPE_T;

    /*
    //	Assign a channel to the target disk device
    */
    lStatus = sys$assign( &device, &sIoChannel, 0, 0);
    if ( !$VMS_STATUS_SUCCESS( lStatus )) return lStatus;

    /*
    //	The following hunk of code locks the target volume
    */
    memset( &sFib, 0, sizeof( struct fibdef1 ));
    sFib.fib$w_cntrlfunc = FIB$C_LOCK_VOL;

    lStatus = sys$qiow( 0, sIoChannel, IO$_ACPCONTROL,
	&qIosb[0],  0, 0, &dFib, 0, 0, 0, 0, 0);
    if ( !$VMS_STATUS_SUCCESS( lStatus )) return lStatus;
    if ( !$VMS_STATUS_SUCCESS( qIosb[0] )) return qIosb[0];

    /*
    //	Perform the operation(s) sensitive to the file
    //	structure here. 
    //
    //	Alternatively, STEP to this point in the debugger,
    //	and test volume access from another process.
    */

    /*
    //	The following hunk of code unlocks the target volume
    */
    memset( &sFib, 0, sizeof( struct fibdef1 ));
    sFib.fib$w_cntrlfunc = FIB$C_UNLK_VOL;

    lStatus = sys$qiow( 0, sIoChannel, IO$_ACPCONTROL,
	&qIosb[0],  0, 0, &dFib, 0, 0, 0, 0, 0);
    if ( !$VMS_STATUS_SUCCESS( lStatus )) return lStatus;
    if ( !$VMS_STATUS_SUCCESS( qIosb[0] )) return qIosb[0];

    /*
    //	done...
    */
    return SS$_NORMAL;
    }

