#pragma module QIOEXTREAD "V1.0-000"
/*
// COPYRIGHT (c) 1992-1998 BY
// DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.
// ALL RIGHTS RESERVED.
//
// THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED
// ONLY  IN  ACCORDANCE  OF  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE
// INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER
// COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY
// OTHER PERSON.  NO TITLE TO AND  OWNERSHIP OF THE  SOFTWARE IS  HEREBY
// TRANSFERRED.
//
// THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE
// AND  SHOULD  NOT  BE  CONSTRUED  AS A COMMITMENT BY DIGITAL EQUIPMENT
// CORPORATION.
//
// DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS
// SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.
*/
/*
//++
//  Facility:
//
//      Examples
//
//  Version: V1.0
//
//  Abstract:
//
//      Example of working with the sys$qiow extended read call,
//      with terminators, and with control (escape) sequences.
//      It also shows how to disable command line editing, as
//      well as the echoing of user input.
//
//  Author:
//      Steve Hoffman
//
//  Creation Date:  2-Jan-1992
//
//  Modification History:
//      23-Aprn-1998 Hoffman
//      Improved the comments, tweaked for DEC C.
//--
*/

#include <ctype.h>
#include <descrip.h>
#include <file.h>
#include <iodef.h>
#include <lib$routines.h>
#include <ssdef.h>
#include <starlet.h>
#include <stdio.h>
#include <stsdef.h>
#include <trmdef.h>


#define IL3MAX 5
#define TIMEOUT		10
#define BUFFEROVER	10
#define BUFFERBASE	80
#define BUFFERSIZE	(BUFFERBASE+BUFFEROVER)

/*
//  The following is the core of the example program,
//  the routine that performs the sys$qiow call...
*/
static int Ef;
int ReadWithTimeout( int *IoChan, char *InputP,
  int InputLen, int TimeOutSec )
    {
    int RetStat, FuncCode, ItemListSize;
    struct
	{
	short RetStat;
	short TermOff;
	char TermChar;
	char Reserved;
	char TermLen;
	char EolOff;
	} Iosb;
    struct ItemList3
	{
	short int ItemBufLen;
	short int ItemCode;
	void *ItemBufAdr;
	void *ItemBufRla;
	};
    struct ItemList3 ItemList[IL3MAX];
    int i;
    $DESCRIPTOR( SysCommand, "SYS$COMMAND" );

    /*
    //	Assign a channel to SYS$COMMAND (only) on the first call.
    //  (IoChan is already an address, so we don't need &IoChan here.)
    //  No attempt is made to deassign this channel in this example.
    */
    if (!*IoChan)
	{
	RetStat = sys$assign( &SysCommand, IoChan, 0, 0, 0);
	if (!$VMS_STATUS_SUCCESS( RetStat ))
	    return RetStat;
	RetStat = lib$get_ef( &Ef );
	if (!$VMS_STATUS_SUCCESS( RetStat ))
	    return RetStat;
	}


    /*
    //	Set up the call to QIO by seeing whether a timeout is requested
    */
    i = 0;
    if ( TimeOutSec )
	{
	/*
	//  timeout (in seconds)
	*/
	ItemList[i].ItemCode     = TRM$_TIMEOUT;
	ItemList[i].ItemBufLen   = 0;
	ItemList[i].ItemBufAdr   = (void *) TimeOutSec;
	ItemList[i++].ItemBufRla = NULL;
	}

    ItemList[i].ItemCode     = TRM$_ESCTRMOVR;
    ItemList[i].ItemBufLen   = 0;
    ItemList[i].ItemBufAdr   = BUFFEROVER;
    ItemList[i++].ItemBufRla = NULL;

    ItemList[i].ItemCode     = TRM$_MODIFIERS;
    ItemList[i].ItemBufLen   = 0;
    ItemList[i].ItemBufAdr   = (TRM$M_TM_ESCAPE | TRM$M_TM_TRMNOECHO |
	                       TRM$M_TM_NOFILTR | TRM$M_TM_NORECALL);
    ItemList[i++].ItemBufRla = NULL;

    /*
    //  Unlike most OpenVMS system services, this itemlist
    //  is not a null-terminated array, it is passed with
    //  an argument that indicates the address of the array,
    //  and a second argument that contains the total length
    //  of the itemlist.  Calculate the length here...
    */
    ItemListSize = i * sizeof( struct ItemList3 );

    /*
    //  Create the function code, and call the sys$qiow call.
    */
    FuncCode = IO$_READVBLK | IO$M_EXTEND;
    RetStat = sys$qiow( Ef, IoChan, FuncCode, &Iosb, 0, 0,
	InputP, InputLen, 0, 0, &ItemList, ItemListSize );


    /*
    //  Check for a failure of the sys$qiow via RetStat, of
    //  a specific timeout via Iosb.RetStat, and of any general
    //  failure of the I/O operation via Iosb.RetStat...
    */
    if (!$VMS_STATUS_SUCCESS( RetStat ))
	return RetStat;
    if (Iosb.RetStat == SS$_TIMEOUT ) 
	return Iosb.RetStat;
    if ( !$VMS_STATUS_SUCCESS( Iosb.RetStat ) )
	return Iosb.RetStat;

    /*
    //	The terminator happened to be <CTRL/Z>.  It is time to leave...
    */
    if ( Iosb.TermChar == 0x01a )
	return SS$_ENDOFFILE;

    /*
    //  Now display what we received from the user...
    //  As the terminator is likely to contain characters
    //  that are not printable, check for and convert any
    //  of these characters into a text representation.
    */
    printf("\n");
    printf("The IOSB Contents:\n");
    printf("  sys$qiow status     : 0x0%x\n", (unsigned short) Iosb.RetStat );
    printf("  Terminator offset   : 0x0%x\n", (unsigned short) Iosb.TermOff );
    printf("  Terminator char     : 0x0%x\n", (unsigned char) Iosb.TermChar );
    printf("  Reserved            : 0x0%x\n", (unsigned char) Iosb.Reserved );
    printf("  Terminator length   : 0x0%x\n", (char) Iosb.TermLen );
    printf("  End of line offset  : 0x0%x\n", (char) Iosb.EolOff );
    printf("\n");
    
    printf("The Text Character(s):\n");
    printf("  {%*.*s}\n", Iosb.TermOff, Iosb.TermOff, InputP );
    printf("\n");
    printf("The Terminator Character(s):\n");
    printf("  {" );
    for (i = Iosb.TermOff; i < Iosb.TermOff + Iosb.TermLen; i++)
	if ( isprint(  (int) InputP[i] ))
	     printf( "%c", (unsigned char) InputP[i] );
	else
	     printf("<0x0%x>", (unsigned char) InputP[i] );
    printf("}\n\n");

    return SS$_NORMAL;
    }

/*
//  The following is a simple program that demonstrates the call...
*/
main()
    {
    char UserBuffer[BUFFERSIZE];
    int RetStat;
    int UserChan = 0;

    printf("This program loops until <CTRL/Z> or until timeout...\n");
    printf("No input will be echoed, and no command editing is permitted.\n");
    do 	{
	printf("\n");
	printf("You have %d seconds to enter a string and a terminator:\n",
          TIMEOUT );

	RetStat = ReadWithTimeout( &UserChan, UserBuffer, BUFFERSIZE, TIMEOUT );

	printf("return status: 0x0%x\n", RetStat );
	} while ($VMS_STATUS_SUCCESS( RetStat ) );

    return SS$_NORMAL;
    }
