/*
// COPYRIGHT (c) 1996 BY
// DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.
// ALL RIGHTS RESERVED.
//
// THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED
// ONLY  IN  ACCORDANCE  OF  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE
// INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER
// COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY
// OTHER PERSON.  NO TITLE TO AND  OWNERSHIP OF THE  SOFTWARE IS  HEREBY
// TRANSFERRED.
//
// THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE
// AND  SHOULD  NOT  BE  CONSTRUED  AS A COMMITMENT BY DIGITAL EQUIPMENT
// CORPORATION.
//
// DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS
// SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.
*/

/*
//++
//  Facility:
//
//      Examples
//
//  Version: V1.0
//
//  Abstract:
//
//      Example of working with ANSI C <stdarg.h> variable argument lists
//
//	$ CC/DECC[/[NO]DEBUG[/NOOPT]] STDARG_PUTMSG
//	$ LINK [/[NO]DEBUG] STDARG_PUTMSG
//	$ RUN [/[NO]DEBUG] STDARG_PUTMSG
//
//
//  Author:
//      Steve Hoffman
//      hoffman@xdelta.zko.dec.com
//
//  Creation Date:  13-Dec-1996
//
//  Modification History:
//	    Hoffman    17-Dec-1996 
//	    Added the putmsg support
//
//--
*/
#include <descrip.h>
#include <lib$routines.h>
#include <libdef.h>
#include <rmsdef.h>
#include <ssdef.h>
#include <starlet.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stsdef.h>

/*
//  The following is a simple-minded "main" program, which
//  exists solely to call the VarArgListRtn() routine.
*/
main()
    {
    int RetStat;
    int VarArgListRtn( int arg1,... );
    int VarArgPutMsg( int arg1,... );
    $DESCRIPTOR( AmbKeyAS, "<bogus>");

    /*
    // Call the example routine...
    */
    RetStat = VarArgListRtn( 1, 2, 3, 4 );
    if (!$VMS_STATUS_SUCCESS( RetStat )) return RetStat;
    RetStat = VarArgListRtn( 5 );
    if (!$VMS_STATUS_SUCCESS( RetStat )) return RetStat;
    RetStat = VarArgListRtn( 6, 7, 8, 9, 10 );
    if (!$VMS_STATUS_SUCCESS( RetStat )) return RetStat;

    RetStat = VarArgPutMsg( SS$_NORMAL  );
    if (!$VMS_STATUS_SUCCESS( RetStat )) return RetStat;
    RetStat = VarArgPutMsg( RMS$_FNF );
    if (!$VMS_STATUS_SUCCESS( RetStat )) return RetStat;
    RetStat = VarArgPutMsg( LIB$_AMBKEY, (void *) &AmbKeyAS );
    if (!$VMS_STATUS_SUCCESS( RetStat )) return RetStat;

    /*
    //  bye-bye!
    */
    return SS$_NORMAL;
    }

/*
//  The following routine has one fixed argument, followed by
//  a caller-specified variable number of arguments...
*/
int VarArgListRtn( int FixedArg,... )
    {
    int RetStat;
    va_list ap;
    int ArgCount, x;

    /*
    // Set the variable-length processing up, then call the (DEC C
    // specific) routine to get the total number of arguments...
    */
    va_start( ap, FixedArg );
    va_count( ArgCount );

    /*
    // We now -- fetching one argument at a time -- have enough
    // to output the parameter list...
    */
    printf( "The call was: \n");
    printf( "  VarArgListRtn( %d", FixedArg, ArgCount );
    for ( x = 1; x < ArgCount; x++ )
        printf( ", %d", va_arg( ap, int ));
    printf( " );\n" );

    /*
    //  Finish up, and bail out...
    */
    va_end( ap );
    return SS$_NORMAL;
    }

/*
//  The following routine has one fixed argument, followed by
//  a caller-specified variable number of arguments...
*/
int VarArgPutMsg( int FixedArg,... )
    {
    int RetStat;
    va_list ap;
    int ArgCount, x, i, VecArgCount;
    int PutMsgArgArray[256];

    VecArgCount = i = 0;

    /*
    // Set the variable-length processing up, then call the (DEC C
    // specific) routine to get the total number of arguments...
    */
    va_start( ap, FixedArg );
    va_count( ArgCount );

    PutMsgArgArray[i++] = (0x0F << 16);

    /*
    // We now -- fetching one argument at a time -- have enough
    // to process the rest of the argument list -- exactly how
    // we process the rest of the list depends highly on the
    // particular facility code number of the message, as the
    // sys$putmsg call is sensitive to this, as the argument
    // vector is special-cased for RMS and SYSTEM messages...
    */
    PutMsgArgArray[i++] = FixedArg;
    switch ( $VMS_STATUS_FAC_NO( FixedArg ))
	{
	case SYSTEM$_FACILITY:
	    /*
	    //	System messages always have no arguments, and
	    //  no count entry in the array.
	    */
	    VecArgCount = 1;
	    break;
	case RMS$_FACILITY:
	    /*
	    //	RMS messages have (at most) a single argument,
	    //  and no count entry in the array.
	    */
	    VecArgCount = 2;
	    break;
	default:
	    /*
	    //	All other facilities can have a variable number of
	    //  arguments, and all get an entry containing the count.
	    */
	    PutMsgArgArray[i++] = (0x0F << 16) + ArgCount - 1;
	    VecArgCount = ArgCount + 1;
	    break;
	}
    for ( x = 1; x < ArgCount; x++ )
        PutMsgArgArray[i++] = va_arg( ap, int );

    /*
    //  Finish up, polish up the vector, putmsg, and bail out...
    */
    va_end( ap );
    PutMsgArgArray[0] += VecArgCount;
    RetStat = sys$putmsg( PutMsgArgArray, 0, 0, 0 );
    if (!$VMS_STATUS_SUCCESS( RetStat )) return RetStat;

    return SS$_NORMAL;
    }
