
/*
** Copyright 2001 by Compaq Computer Corporation
*/

/*
**++
**  Facility:
**
**      Examples
**
**  Version: V1.0
**
**  Abstract:
**
**      Example of working with the LMF calls.
**      This example program checks for a hardwired product license.
**      If the license is not available, an error is signaled.
**
**  Author:
**      Steve Hoffman
**
**  Creation Date:  1-Jan-1990
**
**  Modification History:
**                  15-Sep-1994  Hoffman
**                  Miscellaneous updates, added signal handler
**                  11-Jun-2001  Hoffman
**                  Miscellaneous updates
**--
*/

/*
//  The following four definitions allow any product to be checked:
//  Bogus product licenses should engender license-related errors.
//  Check for existing, valid product licenses should be accepted.
*/
#define	PRODUCT		"C"
#define	PRODUCER	"DEC"
#define	PROD_VERSION	0x010000
#define PROD_DATE	"1-JAN-1988"

/*
/* Definitions required to get off the ground:  (note the LMF stuff
/* does not yet have a C header file...)
*/
#include <descrip.h>
#include <lib$routines.h>
#include <lmfdef.h>
#include <signal.h>
#include <ssdef.h>
#include <starlet.h>
#include <stdio.h>
#include <stsdef.h>

/* until these calls are in starlet.h */
int sys$lookup_license( void *, void *, void *);
int sys$grant_license( void *, void *, void *, void *);

int lmf()
    {
    int retstat;
    /*
    // the following may eventually be needed for full sys$lookup_license()
    // support.  Right now they're just extra baggage.
    //
    //    int context = 0;
    //    int flags = LMF$M_RETURN_FAILURES | LMF$M_BROADCAST_FAILURES;
    */
    $DESCRIPTOR( product, PRODUCT );
    $DESCRIPTOR( producer, PRODUCER );
    $DESCRIPTOR( prod_date, PROD_DATE );
    int prod_version = PROD_VERSION;
    char bintim[8];
    int n;
    struct itmlst_3
	{
	short buflen;
	short itmcod;
	char *bufadr;
	short *bufrla;
	} itmlst[3];

    /*
    // Fill in the item list required by the call.  Includes the
    // required final, zero-filled, block.
    */
    n = 0;
    itmlst[n].buflen = 8;
    itmlst[n].itmcod = LMF$_PROD_DATE;
    itmlst[n].bufadr = (char *) bintim;
    itmlst[n++].bufrla = NULL;

    itmlst[n].buflen = 4;
    itmlst[n].itmcod = LMF$_PROD_VERSION;
    itmlst[n].bufadr = (char *) &prod_version;
    itmlst[n++].bufrla = NULL;

    itmlst[n].buflen = 0;
    itmlst[n].itmcod = 0;
    itmlst[n].bufadr = NULL;
    itmlst[n++].bufrla = NULL;

    /*
    // convert the time from ASCII to the internal quadword format.
    */
    retstat = sys$bintim( &prod_date, bintim );
    if ( retstat != SS$_NORMAL )
	lib$signal( retstat );

    /*
    // The LMF signals on errors.
    */
    VAXC$ESTABLISH( lib$sig_to_ret );
    retstat = sys$lookup_license( &product, itmlst, &producer );

    return retstat;
    }

/*
// Here's what a really bogus main program looks like:
*/
main()
    {
    int lmf();
    int retstat;

    printf("Checking for product %s %s license.\n", PRODUCER, PRODUCT );

    retstat = lmf();
    switch( retstat )
	{
	case SS$_NOLICENSE:
	case 0x0a948af4:    /* "LICENSE$_NOAUTH" */
	    printf("Product is not currently licensed!\n" );
	    break;
	case SS$_NORMAL:
	    printf("Product is reported as licensed!\n" );
	    break;
	default:
	    printf("Product sys$lookup_license error!\n");
	    printf("...return status was: 0x0%x.\n", retstat );
	    break;
	}

    printf("End checking for product %s %s license.\n", PRODUCER, PRODUCT );
    return SS$_NORMAL;
    }

